? character-improvements.diff
Index: characters.lisp
===================================================================
RCS file: /project/cxml/cvsroot/cxml/xml/characters.lisp,v
retrieving revision 1.2
diff -u -r1.2 characters.lisp
--- characters.lisp	28 Nov 2005 22:33:47 -0000	1.2
+++ characters.lisp	13 Jun 2006 16:16:36 -0000
@@ -109,8 +109,54 @@
       (#x0640 #x0640) (#x0E46 #x0E46) (#x0EC6 #x0EC6) (#x3005 #x3005)
       (#x3031 #x3035) (#x309D #x309E) (#x30FC #x30FE)))
 
+(defparameter *all-special-ranges*
+  (let ((sorted (sort (concatenate 'simple-vector
+				   *base-char-ranges*
+				   *ideographic-ranges*
+				   *digit-ranges*
+				   *combining-char-ranges*
+				   *extender-ranges*)
+		      #'<
+		      :key #'car)))
+    (loop
+	for last-low = 0 then low
+	for last-high = 0 then high
+	for (low high) across sorted
+	do (if (or (< low last-low)
+		   (< low last-high)
+		   (> low high)
+		   (> last-high high))
+	       (error
+		"Attempt to combine character ranges into single sorted range failed:
+	~a"
+		sorted)))
+    sorted))
+
+
+(defun rune-in-range-p (rune range-vector)
+  (declare (type simple-vector range-vector))
+  ;;we were always dealing with a sorted vector... bin search it
+  (let ((code (rune-code rune)))
+    (loop with start = 0 
+	  with end = (length range-vector)
+	  while (< start end)
+	  for mid-index = (+ start (floor (- end start) 2))
+	  for (mid-item-low mid-item-high) = (aref range-vector mid-index)
+	  if (< mid-item-high code) do (setf start (1+ mid-index))
+	  else if (< code mid-item-low) do (setf end mid-index)
+	  else do (return T)
+	  finally (return nil))))
+
+(defun rune-name-char-p (rune)
+  (or (eql rune #/.)
+      (eql rune #/-)
+      (eql rune #/_)
+      (eql rune #/:)
+      (rune-in-range-p rune *all-special-ranges*)))
+
+
 (defun valid-name-p (rod)
-  (and (not (zerop (length rod)))
+  (and (plusp (length rod))
        (let ((initial (elt rod 0)))
          (or (rune-in-range-p initial *base-char-ranges*)
              (rune-in-range-p initial *ideographic-ranges*)
@@ -121,25 +167,3 @@
 (defun valid-nmtoken-p (rod)
   (and (not (zerop (length rod)))
        (every #'rune-name-char-p rod)))
-
-(defun rune-name-char-p (rune)
-  (or (rune-in-range-p rune *base-char-ranges*)
-      (rune-in-range-p rune *ideographic-ranges*)
-      (rune-in-range-p rune *digit-ranges*)
-      (eql rune #/.)
-      (eql rune #/-)
-      (eql rune #/_)
-      (eql rune #/:)
-      (rune-in-range-p rune *combining-char-ranges*)
-      (rune-in-range-p rune *extender-ranges*)))
-
-(defun rune-in-range-p (rune range)
-  ;; XXX FIXME, das geht doch besser
-  (let ((code (rune-code rune)))
-    (block nil
-      (map nil (lambda (range)
-                 (when (< code (car range))
-                   (return nil))
-                 (when (<= code (cadr range))
-                   (return t)))
-           range))))
